﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Infragistics.Olap.FlatData;
using System.Linq.Expressions;
using System.Threading;
using System.Threading.Tasks;
using System.Collections.ObjectModel;
using Infragistics.Olap;

namespace WpfApplication1
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        protected XAMPivotFilterHelper<Person> FilterHelper;
        private ManualResetEvent _mre;

        public MainWindow()
        {
            InitializeComponent();
            FilterHelper = new XAMPivotFilterHelper<Person>();
        }

        protected string SortGender(Person x)
        {
            return x.Gender;  
        }

        protected string SortTypeCode(Person x)
        {
            return x.TypeCode;  
        }

        private void PeopleGrid_Loaded(object sender, RoutedEventArgs e)
        {
            PeopleGrid.Loaded -= PeopleGrid_Loaded;
            var zz = (FlatDataSource)FindResource("PersonFlatDataSource");
            FilterHelper.FlatDataSource = (FlatDataSource)FindResource("PersonFlatDataSource");
            
            PeopleGrid.DataSource = FilterHelper.FlatDataSource;
            
            PeopleGrid.DataSource.ResultChanged += FilterHelper.HierarchyChanged;
            

            FilterHelper.FlatDataSource.ItemsSource = GetPeople();
            System.Linq.Expressions.Expression<Func<Person, string>> sortGender = x => SortGender(x);
            System.Linq.Expressions.Expression<Func<Person, string>> sortTypeCode = x => SortTypeCode(x);
            // don't set an expression at index 0, this is the root level

            PeopleGrid.DataSource.DeferredLayoutUpdate = true;

            if (PeopleGrid.DataSource.Filters.Count > 0)
                PeopleGrid.DataSource.Filters.Clear();

            //FilterHelper.FlatDataSource.HierarchyDescriptors[0].LevelDescriptors[0].OrderByKeyExpression = sortGender;
            //FilterHelper.FlatDataSource.HierarchyDescriptors[0].LevelDescriptors[1].OrderByKeyExpression = sortGender;
            //FilterHelper.FlatDataSource.HierarchyDescriptors[1].LevelDescriptors[1].OrderByKeyExpression = sortTypeCode;
            //FilterHelper.FlatDataSource.HierarchyDescriptors[2].LevelDescriptors[1].OrderByKeyExpression = (Expression<Func<Person, decimal>>)(p => p.AgeGroup);
          
            IFilterViewModel pmFilterViewModel = PeopleGrid.DataSource.CreateFilterViewModel(PeopleGrid.DataSource.Cube.Dimensions["BirthDate"].Hierarchies.First());
            PeopleGrid.DataSource.Filters.Add(pmFilterViewModel);

            FilterBirthDate();
            FilterAgeGroup();

            // process the expansion state
            IFilterViewModel fvm = FilterHelper.GetFilterViewModel("TypeCode");
            fvm.LoadFilterMembersCompleted += (s, f) => _mre.Set();

            foreach (Infragistics.Olap.Data.IFilterMember member in fvm.FilterMembers)
                FilterHelper.ExpandHierarchy(member, true);

            fvm = FilterHelper.GetFilterViewModel("Gender");
            fvm.LoadFilterMembersCompleted += (s, f) => _mre.Set();

            foreach (Infragistics.Olap.Data.IFilterMember member in fvm.FilterMembers)
                FilterHelper.ExpandHierarchy(member, true);

            
            PeopleGrid.DataSource.DeferredLayoutUpdate = false;
            PeopleGrid.DataSource.RefreshGrid();
        }

        public ObservableCollection<Person> GetPeople()
        {
            List<Person> people = new List<Person>();
            people.Add(new Person { Gender = "M", TypeCode = "D", AgeGroup = 2.5m, BirthDate = "12/1/1988",  Income = 100 });
            people.Add(new Person { Gender = "M", TypeCode = "B", AgeGroup = 3m, BirthDate = "12/1/1988", Income = 112 });
            people.Add(new Person { Gender = "M", TypeCode = "C", AgeGroup = 11m, BirthDate = "12/1/1988", Income = 95 });
            people.Add(new Person { Gender = "M", TypeCode = "A", AgeGroup = 7m, BirthDate = "12/1/1988", Income = 130 });

            people.Add(new Person { Gender = "F", TypeCode = "A", AgeGroup = 4.999999999m, BirthDate = "12/1/1988", Income = 113 });
            people.Add(new Person { Gender = "F", TypeCode = "B", AgeGroup = 12m, BirthDate = "12/1/1988", Income = 1 });
            people.Add(new Person { Gender = "F", TypeCode = "D", AgeGroup = 1m, BirthDate = "12/6/1988", Income = 115 });
            people.Add(new Person { Gender = "F", TypeCode = "E", AgeGroup = 9.5m, BirthDate = "12/5/1988", Income = 86 });
            return new ObservableCollection<Person>(people.OrderBy(x => x.Gender).ThenBy(x => x.AgeGroup).ThenBy(x => x.TypeCode));
            //return new ObservableCollection<Person>(people);
        }

        protected void FilterBirthDate()
        {
            FilterHelper.FilterOption("BirthDate", "All", true);

            //FilterHelper.FilterOption("BirthDate", "12/1/1988", true);
            FilterHelper.FlatDataSource.RefreshGrid();
        }

        protected void FilterAgeGroup()
        {
            FilterHelper.FilterOption("AgeGroup", "All", false);

            FilterHelper.FilterOption("AgeGroup", "12", true);
            FilterHelper.FilterOption("AgeGroup", "1",true);
            
            FilterHelper.FlatDataSource.RefreshGrid();
        }

        private void PeopleGrid_CellDoubleClicked(object sender, Infragistics.Controls.Grids.PivotCellClickedEventArgs e)
        {

        }
    }

    public class Person
    {
        public string Gender { get; set; }
        public string TypeCode { get; set; }
        public decimal AgeGroup { get; set; }
        public string BirthDate { get; set; }
        public decimal Income { get; set; }
    }
}
